<?php

require_once('class.category.selector.php');

/**
 * Manage the category selectors.
 *
 * @property        SLPTagalong                     $addon
 * @property-read	array							$categoryDropDowns 			The category drop downs.
 * @property-read	int								$node_level					Which level of the nested drop down tree are we processing?
 * @property        SLPTagalong_Category_Selector[] $selectors                  named array, key = slug, value = selector object
 *
 * @package StoreLocatorPlus\Tagalong\Category\Selector\Manager
 * @author Lance Cleveland <lance@charlestonsw.com>
 * @copyright 2015 Charleston Software Associates, LLC
 */
class SLPTagalong_Category_Selector_Manager extends SLPlus_BaseClass_Object {
    public  $addon;
    private $categoryDropDowns = array();
    private $node_level = 0;
    public  $selectors;

    /**
     * Initialize the manager.
     */
    function __construct( $options = array() ) {
        parent::__construct( $options );
        $this->create_default_selectors();
    }

    /**
     * Create the default selectors.
     */
    private function create_default_selectors() {

        // No
        $this->selectors['none'] =
            new SLPTagalong_Category_Selector(array(
                'label'         => __('None','slp-tagalong'),
                'value'         =>'none',
                'selected'      => ($this->addon->options['show_cats_on_search'] === '')
            ));

        // Single
        $this->selectors['single'] =
            new SLPTagalong_Category_Selector(array(
                'label'         => __('Single','slp-tagalong'),
                'value'         => 'single',
                'selected'      => ($this->addon->options['show_cats_on_search'] === 'single'),
                'ui_builder'    => array( $this , 'create_string_for_single_selector' )
            ));

        // Cascading
        $this->selectors['cascade'] =
            new SLPTagalong_Category_Selector(array(
                'label'         => __('Cascading','slp-tagalong'),
                'value'         => 'cascade',
                'selected'      => ($this->addon->options['show_cats_on_search'] === 'cascade'),
                'ui_builder'    => array( $this , 'create_string_for_cascade_selector' )
            ));


        /**
         * Filter to augment the default Tagalong category selector types.
         *
         * @params  SLPTagalong_Category_Selector[]  $this->selectors
         *
         * @return SLPTagalong_Category_Selector[]
         */
        $this->selectors = apply_filters( 'slp_tagalong_default_selectors' , $this->selectors );
    }

    /**
     * Create a drop down object for all items with a parent category as specified.
     *
     * Recursive, calls the same method for each child.
     *
     * @param string $parent_cat the parent category (int)
     * @param mixed $grandparent_cat the grandparent category (int) or null
     * @return mixed
     */
    private function create_dropdown_for_category($parent_cat,$grandparent_cat=null) {
        if (!ctype_digit($parent_cat)) { return array(); }

        $categories = get_categories(
            array(
                'hierarchical'      => false,
                'hide_empty'        => false,
                'orderby'           => 'name',
                'parent'            => $parent_cat,
                'taxonomy'          => SLPLUS::locationTaxonomy
            )
        );

        if (count($categories)<=0) { return array(); }

        $dropdownItems = array();
        $dropdownItems[] =
            array(
                'label' => $this->addon->options['show_option_all'],
                'value' => ''
            );
        foreach ($categories as $category) {
            $dropdownItems[] =
                array(
                    'label' => $category->name,
                    'value' => $category->term_id
                );
            $this->create_dropdown_for_category($category->term_id,$parent_cat);
        }
        $this->categoryDropDowns[] = array(
            'grandparent' => $grandparent_cat,
            'parent'    => $parent_cat,
            'id'        => 'catsel_'.$parent_cat,
            'name'      => 'catsel_'.$parent_cat,
            'items'     => $dropdownItems,
            'onchange'  =>
                "jQuery('#children_of_{$parent_cat}').children('div.category_selector.child').hide();" .
                "childDD='#children_of_'+jQuery('option:selected',this).val();" .
                "jQuery(childDD).show();" .
                "jQuery(childDD+' option:selected').prop('selected',false);" .
                "jQuery(childDD+' option:first').prop('selected','selected');" .
                "if (jQuery('option:selected',this).val()!=''){jQuery('#cat').val(jQuery('option:selected',this).val());}" .
                "else{jQuery('#cat').val(jQuery('#catsel_{$grandparent_cat} option:selected').val());}"
        );
    }

    /**
     * Create a cascading drop down array for location categories.
     *
     */
    private function create_string_cascading_category_dropdown() {

        // Build the category drop down object array, recursive.
        //
        $this->create_dropdown_for_category('0');

        // Create the drop down HTML for each level
        //
        if (count($this->categoryDropDowns) > 0) {
            $this->categoryDropDowns = array_reverse($this->categoryDropDowns);
            $HTML =
                "<div id='tagalong_cascade_dropdowns'>" .
                '<input type="hidden" id="cat" name="cat" value=""/>'       .
                $this->create_string_nested_dropdown_divs($this->categoryDropDowns[0]['parent']) .
                '</div>'
            ;
        } else {
            $HTML = '';
        }

        return $HTML;
    }

    /**
     * Create the div-wrapped HTML with label for the category selector.
     *
     * @param  string $dropdown_html
     * @param  int    $term_id
     * @return string
     */
    public function create_string_dropdown_div( $dropdown_html , $term_id = 0 ) {
        if ( empty( $dropdown_html ) ) { return ''; }

        if ( ! empty( $this->addon->options['label_category'] ) ) {
            $label_html =
                '<label for="cat">' .
                    $this->slplus->WPML->get_text( 'TAGALONG-label_category' , $this->addon->options['label_category'] ,  'slp-tagalong' ) .
                '</label>'
                ;
        } else {
            $label_html = '<label for="cat">&nbsp;</label>';
        }

        $term_slug = ( $term_id > 0 ) ? '_' . $this->addon->stores_taxonomy->get_term( $term_id , 'slug' ) : '';

        return
            "<div id='tagalong_category_selector{$term_slug}' class='tagalong_category_selector tagalong_term${term_slug} search_item'>" .
                $label_html .
                $dropdown_html .
            '</div>'
            ;

    }

    /**
     * Create the HTML output string for the single selector type.
     *
     * @return string
     */
    private function create_string_for_cascade_selector() {
        return $this->create_string_dropdown_div( $this->create_string_cascading_category_dropdown() );
    }

    /**
     * Create the HTML output string for the single selector type.
     *
     * Called via the call_user_func() in get_html().
     *
     * @return string
     */
    private function create_string_for_single_selector() {
        return
            $this->create_string_dropdown_div(
                wp_dropdown_categories(
                    array(
                        'echo'              => 0,
                        'hierarchical'      => 1,
                        'depth'             => 99,
                        'hide_empty'        => ( $this->slplus->is_CheckTrue( $this->addon->options['hide_empty'] ) ? 1 : 0 ),
                        'hide_if_empty'     => true,
                        'orderby'           => 'NAME',
                        'show_option_all'   =>
                            $this->slplus->WPML->get_text(
                                'TAGALONG-show_option_all'          ,
                                $this->addon->options['show_option_all']   ,
                                'slp-tagalong'
                            ),
                        'taxonomy'          => SLPLUS::locationTaxonomy
                    )
                )
            );
    }

    /**
     * Create nested divs with the drop down menus within.
     *
     * @param string $parent_category_id (int)
     * @return string
     */
    private function create_string_nested_dropdown_divs($parent_category_id) {
        $this->node_level++;
        $HTML = '';
        foreach ($this->categoryDropDowns as $dropdown) {
            if ($dropdown['parent']===$parent_category_id) {
                $HTML .=
                    "<div id='div_{$dropdown['id']}' name='div_{$dropdown['id']}' class='category_selector parent' >" .
                    $this->slplus->helper->createstring_DropDownMenu(
                        array(
                            'id'        => $dropdown['id'],
                            'name'      => $dropdown['name'],
                            'onchange'  => $dropdown['onchange'],
                            'items'     => $dropdown['items']
                        )
                    ) .
                    '</div>'
                ;
                foreach ($dropdown['items'] as $item) {
                    $HTML .= $this->create_string_nested_dropdown_divs($item['value']);
                }
            }
        }

        if (!empty($HTML)) {
            $parent_or_child = (($this->node_level === 1) ? 'parent':'child');
            $HTML =
                "<div id='children_of_{$parent_category_id}' class='category_selector {$parent_or_child} level_{$this->node_level}'>" .
                $HTML .
                '</div>';
        }

        $this->node_level--;
        return $HTML;
    }

    /**
     * Return the selectors as a named array or object array.
     *
     * @param string                                    $output  ARRAY_A or OBJECT (constants)
     * @return array|SLPTagalong_Category_Selector[]
     */
    public function get_selectors( $output = ARRAY_A ) {
        if ( $output === ARRAY_A ) {
            $return_array = array();
            foreach ($this->selectors as $slug => $selector ) {
                $return_array[] = (array) $selector;
            }
            return $return_array;
        } elseif ( $output === OBJECT ) {
            return $this->selectors;
        }
    }

    /**
     * Return the UI HTML for the named selector.
     *
     * @param $selector_slug
     * @return mixed
     */
    public function get_html( $selector_slug ) {
        do_action( 'slp_tagalong_get_selector_html' );
        if (
            isset( $this->selectors[ $selector_slug ]                   )     &&
            isset( $this->selectors[ $selector_slug ]->ui_builder       )     &&
            is_callable( $this->selectors[ $selector_slug ]->ui_builder )
        ) {
            return call_user_func($this->selectors[$selector_slug]->ui_builder);
        }
        return '';
    }

}