<?php

require_once( SLPLUS_PLUGINDIR . 'include/base_class.object.php' );

/**
 * Class SLP_Option_Manager
 *
 * Methods to help manage the SLP options.
 *
 * @package   StoreLocatorPlus\Options\Manager
 * @author    Lance Cleveland <lance@charlestonsw.com>
 * @copyright 2016 Charleston Software Associates, LLC
 *
 * @since     4.3.20
 *
 * @property        string       $option_slug        Used by the get_wp_option('js') call.
 * @property        string       $option_nojs_slug   Used by the get_wp_option('nojs') call.
 * @property-read   SLP_Option[] $smart_options      New options using the SLP_Option object type.
 * @property        string[]     $text_options       An array of smart option slugs that are text options.
 *
 */
class SLP_Option_Manager extends SLPlus_BaseClass_Object {
    public $option_slug;
    public $option_nojs_slug;
    private $smart_options;
    public $text_options;

    /**
     * Things we do at startup.
     */
    public function initialize() {
        $this->option_slug = SLPLUS_PREFIX . '-options';
        $this->option_nojs_slug = SLPLUS_PREFIX . '-options_nojs';

        $this->create_smart_options();
    }

    /**
     * Create smart option objects.
     *
     */
    private function create_smart_options() {
        require_once( 'class.option.php' );

        // @var SLP_Option[]    $options
        $options[] = new SLP_Option( array( 'slug' => 'instructions', 'is_text' => true,  ) );
        $options[] = new SLP_Option( array( 'slug' => 'invalid_query_message', 'is_text' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_directions', 'is_text' => true, 'use_in_javascript' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_email', 'is_text' => true, 'use_in_javascript' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_fax', 'is_text' => true, 'use_in_javascript' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_hours', 'is_text' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_image', 'is_text' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_phone', 'is_text' => true, 'use_in_javascript' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_radius', 'is_text' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_search', 'is_text' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'label_website', 'is_text' => true, 'use_in_javascript' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'message_no_results', 'is_text' => true, 'use_in_javascript' => true ) );
        $options[] = new SLP_Option( array( 'slug' => 'message_no_api_key', 'is_text' => true, 'use_in_javascript' => true ) );

        // Set associative array for smart_options
        // @var SLP_Option $option
        foreach ( $options as $option ) {
            $this->smart_options[ $option->slug ] = $option;
            if ( $option->is_text ) {
                $this->text_options[] = $option->slug;
            }
        }
    }

    /**
     * Return a list of option slugs that are text options.
     *
     * @return string[]
     */
    public function get_text_options() {
        if ( !isset( $this->text_options ) ) {
            foreach ( $this->smart_options as $slug => $option ) {
                if ( $option->is_text ) {
                    $this->text_options[] = $option->slug;
                }
            }
        }
        return $this->text_options;
    }

    /**
     * Delete the Store Locator Plus options from the WordPress options table.
     *
     * Default option name is csl-slplus-options per $this->option_slug.
     *
     * @param string $which_option 'default' , 'js' , 'nojs' or the option name.
     *
     * @return mixed|void
     */
    public function delete_wp_option( $which_option = 'default' ) {
        return delete_option( $this->set_the_slug( $which_option, 'delete' ) );
    }

    /**
     * Set the option slug.
     *
     * @param string $which_option 'js', 'nojs', or the option name
     * @param string $mode         'get' or 'update'
     *
     * @return mixed|null|void
     */
    private function set_the_slug( $which_option, $mode ) {
        switch ( $which_option ) {
            case 'default':
            case 'js':
                $slug_to_fetch = $this->option_slug;
                break;
            case 'nojs':
                $slug_to_fetch = $this->option_nojs_slug;
                break;
            default:
                $slug_to_fetch = $which_option;
                break;
        }

        /**
         * FILTER: slp_option_slug
         *
         * @param   string $slug_to_fetch the name of the wp_options table key to fetch with get_option().
         * @param   string $get_or_update 'get' or 'update
         *
         * @return  string      a modified slug
         */
        return apply_filters( 'slp_option_slug', $slug_to_fetch, $mode );

    }

    /**
     * Things we do once after the plugins are loaded.
     */
    public function initialize_after_plugins_loaded() {
        $this->set_text_string_defaults();
        $this->slp_specific_setup();
    }

    /**
     * Set text string defaults.
     */
    private function set_text_string_defaults() {
        foreach ( $this->get_text_options() as $key ) {

            if ( array_key_exists( $key, $this->slplus->options ) ) {
                $this->slplus->options[ $key ] = $this->get_string_default( $key );

            } elseif ( array_key_exists( $key, $this->slplus->options_nojs ) ) {
                $this->slplus->options_nojs[ $key ] = $this->get_string_default( $key );

            }
        }
    }

    /**
     * Get string defaults.
     *
     * @param string $key key name for string to translate
     *
     * @return string
     */
    private function get_string_default( $key ) {
        $this->slplus->create_object_text_manager();
        $text_to_return = $this->slplus->text_manager->get_option_default( $key );
        if ( empty( $text_to_return ) ) {
            $text_to_return = apply_filters( 'slp_string_default', '', $key );
        }

        return $text_to_return;
    }

    /**
     * Initialize the options properties from the WordPress database.
     */
    public function slp_specific_setup() {

        // Set options defaults to values set in property definition above.
        //
        $this->slplus->options[ 'map_home_icon' ] = SLPLUS_ICONURL . 'box_yellow_home.png';
        $this->slplus->options[ 'map_end_icon' ] = SLPLUS_ICONURL . 'bulb_azure.png';

        // Serialized Options from DB for JS parameters
        //
        $this->slplus->options_default = $this->slplus->options;
        $dbOptions = $this->get_wp_option( 'js' );
        if ( is_array( $dbOptions ) ) {
            array_walk( $dbOptions, array( $this->slplus, 'set_ValidOptions' ) );
        }

        // Map Center Fallback
        //
        $this->slplus->recenter_map();

        // Load serialized options for noJS parameters
        //
        $this->slplus->options_nojs_default = $this->slplus->options_nojs;
        $dbOptions = $this->get_wp_option( 'nojs' );
        if ( is_array( $dbOptions ) ) {
            array_walk( $dbOptions, array( $this->slplus, 'set_ValidOptionsNoJS' ) );
        }
        $this->slplus->javascript_is_forced = $this->slplus->is_CheckTrue( $this->slplus->options_nojs[ 'force_load_js' ] );

        $this->set_to_default_if_empty();
    }

    /**
     * Fetch the Store Locator Plus options from the WordPress options table.
     *
     * Default option name is csl-slplus-options per $this->option_slug.
     *
     * @param string $which_option 'default' , 'js' , 'nojs' or the option name.
     * @param mixed  $default
     *
     * @return mixed|void
     */
    public function get_wp_option( $which_option = 'default', $default = false ) {
        return get_option( $this->set_the_slug( $which_option, 'get' ), $default );
    }

    /**
     * If option is empty then set it to the default.
     *
     * Defaults are in the slplus->defaults property.
     *
     * options              defaults
     * bubblelayout         bubblelayout
     *                      layout
     *                      maplayout
     * results_layout       resultslayout
     *                      searchlayout
     */
    private function set_to_default_if_empty() {
        foreach ( $this->slplus->defaults as $name => $value ) {
            if ( empty( $this->slplus->options[ $name ] ) && ( !empty( $value ) ) ) {
                $this->slplus->options[ $name ] = $value;
            }
        }
    }

    /**
     * Set an option in an array only if the key already exists, for empty values set to default.
     *
     * @param mixed  $val - the value of a form var
     * @param string $key - the key for that form var
     * @param        $option_array
     * @param        $default_array
     */
    public function set_valid_option( $val, $key, &$option_array, $default_array ) {
        if ( array_key_exists( $key, $option_array ) ) {
            if ( is_numeric( $val ) || is_bool( $val ) || !empty( $val ) ) {
                $option_array[ $key ] = stripslashes_deep( $val );
            } else {
                if ( isset( $this->smart_options[ $key ] ) ) {
                    $option_array[ $key ] = $this->smart_options[ $key ]->default_if_empty;
                } else {
                    $option_array[ $key ] = $default_array[ $key ];
                }
            }
        }
    }

    /**
     * Update the WordPress option.
     *
     * @param string $which_option  'default' , 'js' , 'nojs' or the option name.
     * @param mixed  $option_values values to be stored
     *
     * @return mixed
     */
    public function update_wp_option( $which_option = 'default', $option_values = NULL ) {
        if ( is_null( $option_values ) ) {
            switch ( $which_option ) {
                case 'default':
                case 'js':
                    $option_values = $this->slplus->options;
                    break;
                case 'nojs':
                    $option_values = $this->slplus->options_nojs;
                    break;
                default:
                    break;
            }
        }

        return update_option( $this->set_the_slug( $which_option, 'update' ), $option_values );
    }

}
